# frozen_string_literal: true

# #Medical Network Professional Claims v3
#
# Reference for developers
#
# OpenAPI spec version: v3
#
# Generated by: https://github.com/swagger-api/swagger-codegen.git
# Swagger Codegen version: 2.4.19
#

require 'date'

module ChangeHealthcare
  module ProfessionalClaims
    module SwaggerClient
      class DrugIdentification
        # Can only set one of linkSequenceNumber and pharmacyPrescriptionNumber but not both
        attr_accessor :link_sequence_number

        # Allowed Values are:'F2' International Unit, 'GR' Gram, 'ME' Milligram, 'ML' Milliliter, 'UN' Unit
        attr_accessor :measurement_unit_code

        attr_accessor :national_drug_code, :national_drug_unit_count

        # Can only set one of linkSequenceNumber and pharmacyPrescriptionNumber but not both
        attr_accessor :pharmacy_prescription_number

        # Allowed Values are:'EN' EAN/UCC - 13, 'EO' EAN/UCC - 8, 'HI' HIBC (Health Care Industry Bar Code) Supplier Labeling Standard Primary Data Message, 'N4' National Drug Code in 5-4-2 Format, 'ON' Customer Order Number, 'UK' GTIN 14-digit Data Structure, 'UP' UCC - 12
        attr_accessor :service_id_qualifier

        class EnumAttributeValidator
          attr_reader :datatype, :allowable_values

          def initialize(datatype, allowable_values)
            @allowable_values = allowable_values.map do |value|
              case datatype.to_s
              when /Integer/i
                value.to_i
              when /Float/i
                value.to_f
              else
                value
              end
            end
          end

          def valid?(value)
            !value || allowable_values.include?(value)
          end
        end

        # Attribute mapping from ruby-style variable name to JSON key.
        def self.attribute_map
          {
            link_sequence_number: :linkSequenceNumber,
            measurement_unit_code: :measurementUnitCode,
            national_drug_code: :nationalDrugCode,
            national_drug_unit_count: :nationalDrugUnitCount,
            pharmacy_prescription_number: :pharmacyPrescriptionNumber,
            service_id_qualifier: :serviceIdQualifier
          }
        end

        # Attribute type mapping.
        def self.swagger_types
          {
            link_sequence_number: :String,
            measurement_unit_code: :String,
            national_drug_code: :String,
            national_drug_unit_count: :String,
            pharmacy_prescription_number: :String,
            service_id_qualifier: :String
          }
        end

        # Initializes the object
        # @param [Hash] attributes Model attributes in the form of hash
        def initialize(attributes = {})
          return unless attributes.is_a?(Hash)

          # convert string to symbol for hash key
          attributes = attributes.transform_keys(&:to_sym)

          self.link_sequence_number = attributes[:linkSequenceNumber] if attributes.key?(:linkSequenceNumber)

          self.measurement_unit_code = attributes[:measurementUnitCode] if attributes.key?(:measurementUnitCode)

          self.national_drug_code = attributes[:nationalDrugCode] if attributes.key?(:nationalDrugCode)

          self.national_drug_unit_count = attributes[:nationalDrugUnitCount] if attributes.key?(:nationalDrugUnitCount)

          self.pharmacy_prescription_number = attributes[:pharmacyPrescriptionNumber] if attributes.key?(:pharmacyPrescriptionNumber)

          self.service_id_qualifier = attributes[:serviceIdQualifier] if attributes.key?(:serviceIdQualifier)
        end

        # Show invalid properties with the reasons. Usually used together with valid?
        # @return Array for valid properties with the reasons
        def list_invalid_properties
          []
        end

        # Check to see if the all the properties in the model are valid
        # @return true if the model is valid
        def valid?
          measurement_unit_code_validator = EnumAttributeValidator.new('String', %w[F2 GR ME ML UN])
          return false unless measurement_unit_code_validator.valid?(@measurement_unit_code)

          service_id_qualifier_validator = EnumAttributeValidator.new('String', %w[EN EO HI N4 ON UK UP])
          return false unless service_id_qualifier_validator.valid?(@service_id_qualifier)

          true
        end

        # Custom attribute writer method checking allowed values (enum).
        # @param [Object] measurement_unit_code Object to be assigned
        def measurement_unit_code=(measurement_unit_code)
          validator = EnumAttributeValidator.new('String', %w[F2 GR ME ML UN])
          unless validator.valid?(measurement_unit_code)
            raise ArgumentError,
                  %(invalid value for "measurement_unit_code", must be one of #{validator.allowable_values}.)
          end

          @measurement_unit_code = measurement_unit_code
        end

        # Custom attribute writer method checking allowed values (enum).
        # @param [Object] service_id_qualifier Object to be assigned
        def service_id_qualifier=(service_id_qualifier)
          validator = EnumAttributeValidator.new('String', %w[EN EO HI N4 ON UK UP])
          unless validator.valid?(service_id_qualifier)
            raise ArgumentError,
                  %(invalid value for "service_id_qualifier", must be one of #{validator.allowable_values}.)
          end

          @service_id_qualifier = service_id_qualifier
        end

        # Checks equality by comparing each attribute.
        # @param [Object] Object to be compared
        def ==(other)
          return true if equal?(other)

          self.class == other.class &&
            link_sequence_number == other.link_sequence_number &&
            measurement_unit_code == other.measurement_unit_code &&
            national_drug_code == other.national_drug_code &&
            national_drug_unit_count == other.national_drug_unit_count &&
            pharmacy_prescription_number == other.pharmacy_prescription_number &&
            service_id_qualifier == other.service_id_qualifier
        end

        # @see the `==` method
        # @param [Object] Object to be compared
        def eql?(other)
          self == other
        end

        # Calculates hash code according to all attributes.
        # @return [Fixnum] Hash code
        def hash
          [link_sequence_number, measurement_unit_code, national_drug_code, national_drug_unit_count,
           pharmacy_prescription_number, service_id_qualifier].hash
        end

        # Builds the object from hash
        # @param [Hash] attributes Model attributes in the form of hash
        # @return [Object] Returns the model itself
        def build_from_hash(attributes)
          return nil unless attributes.is_a?(Hash)

          self.class.swagger_types.each_pair do |key, type|
            if type =~ /\AArray<(.*)>/i
              # check to ensure the input is an array given that the attribute
              # is documented as an array but the input is not
              if attributes[self.class.attribute_map[key]].is_a?(Array)
                send("#{key}=", attributes[self.class.attribute_map[key]].map do |v|
                                  _deserialize(Regexp.last_match(1), v)
                                end)
              end
            elsif !attributes[self.class.attribute_map[key]].nil?
              send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
            end
          end

          self
        end

        # Deserializes the data based on type
        # @param string type Data type
        # @param string value Value to be deserialized
        # @return [Object] Deserialized data
        def _deserialize(type, value)
          case type.to_sym
          when :DateTime
            DateTime.parse(value)
          when :Date
            Date.parse(value)
          when :String
            value.to_s
          when :Integer
            value.to_i
          when :Float
            value.to_f
          when :BOOLEAN
            if value.to_s =~ /\A(true|t|yes|y|1)\z/i
              true
            else
              false
            end
          when :Object
            # generic object (usually a Hash), return directly
            value
          when /\AArray<(?<inner_type>.+)>\z/
            inner_type = Regexp.last_match[:inner_type]
            value.map { |v| _deserialize(inner_type, v) }
          when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
            k_type = Regexp.last_match[:k_type]
            v_type = Regexp.last_match[:v_type]
            {}.tap do |hash|
              value.each do |k, v|
                hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
              end
            end
          else # model
            temp_model = SwaggerClient.const_get(type).new
            temp_model.build_from_hash(value)
          end
        end

        # Returns the string representation of the object
        # @return [String] String presentation of the object
        def to_s
          to_hash.to_s
        end

        # to_body is an alias to to_hash (backward compatibility)
        # @return [Hash] Returns the object in the form of hash
        def to_body
          to_hash
        end

        # Returns the object in the form of hash
        # @return [Hash] Returns the object in the form of hash
        def to_hash
          hash = {}
          self.class.attribute_map.each_pair do |attr, param|
            value = send(attr)
            next if value.nil?

            hash[param] = _to_hash(value)
          end
          hash
        end

        # Outputs non-array value in the form of hash
        # For object, use to_hash. Otherwise, just return the value
        # @param [Object] value Any valid value
        # @return [Hash] Returns the value in the form of hash
        def _to_hash(value)
          if value.is_a?(Array)
            value.compact.map { |v| _to_hash(v) }
          elsif value.is_a?(Hash)
            {}.tap do |hash|
              value.each { |k, v| hash[k] = _to_hash(v) }
            end
          elsif value.respond_to? :to_hash
            value.to_hash
          else
            value
          end
        end
      end
    end
  end
end
